// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of molview_dataserver.msglib;

/// Data types that can be used in data tables
///
/// These data types are similar to the data types that can be stored in SQLite.
enum DataType {
  /// Mathematical integer
  integer,

  /// Real number
  real,

  /// Text string
  text
}

/// General table rejection reasons
///
/// If the reason for rejection is not in this enumeration, the enumeration
/// should be extended.
enum TableRejectionReason {
  /// No reason to reject
  none,

  /// Unkown reason to reject
  unkown,

  /// All table/database names should be unique within the provider.
  duplicateTable,

  /// All table fields must be distinctive.
  duplicateField,

  /// The field data type should match the data type used by fields with the
  /// same data class in previously registered tables.
  illegalFieldDataType,

  /// Keys must have the same canonicality as keys in previously registered
  /// tables.
  illegalKeyCanonicality,

  /// Key relations (one-to-one, one-to-many, etc.) should match the ones used
  /// by previously registered tables.
  incompatibleRelation
}

/// Superclass for fields and keys
abstract class TableField {
  /// Data type
  final DataType dataType;

  /// Data class
  ///
  /// The data class decribes what kind of data is stored in this field.
  /// Standard class strings are defined in this library and should be used when
  /// possible. The class string is used to identify the same data fields across
  /// separate tables.
  final String dataClass;

  /// Constructor
  const TableField(this.dataType, this.dataClass);
}

/// Field data
class FieldData {
  /// Field data class
  final String dataClass;

  /// Field value
  final dynamic value;

  /// Constructor
  FieldData(this.dataClass, this.value);

  /// Construct from PDS.
  factory FieldData.fromPds(Map<String, dynamic> data) {
    return new FieldData(data['class'] as String, data['value']);
  }

  /// Convert to PDS.
  Map<String, dynamic> toPds() {
    return {'class': dataClass, 'value': pdsFromDynamic(value)};
  }
}
