// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of molview_dataserver.msglib;

/// Key relational type
///
/// When a key is included in a table descriptor, the DataServer assumes it can
/// use that table to retrieve this key. Many-to-one keys can be stored on both
/// sides of the relation, in the parent table (one-to-many), and in the child
/// table (many-to-one).
enum KeyRelationType {
  /// This key is unique
  /// (each unique record has a unique key value)
  oneToOne,

  /// This key can be linked to many records
  /// (many records exist with the same key value)
  manyToOne,

  /// This record can be linked with many keys
  /// (one record has many keys that are unique in the table)
  oneToMany,

  /// This record can be linked with many keys that can be linked to many
  /// records.
  /// (one record has many keys that can also be linked to other records)
  manyToMany
}

/// Table key
///
/// Extends [TableField] with a [KeyRelationType].
class TableKey extends TableField {
  /// Relation that is described by this key.
  final KeyRelationType relation;

  /// Constructor
  const TableKey(DataType dataType, String dataClass, this.relation)
      : super(dataType, dataClass);

  /// Construct from PDS.
  factory TableKey.fromPds(Map<String, dynamic> data) {
    return new TableKey(
        DataType.values[data['type'] as int],
        data['class'] as String,
        KeyRelationType.values[data['relation'] as int]);
  }

  /// Convert to PDS.
  Map<String, dynamic> toPds() {
    return {
      'type': dataType.index,
      'class': dataClass,
      'relation': relation.index
    };
  }
}

/// Primary key
///
/// Plain wrapper of [TableField]
class PrimaryKey extends TableField {
  /// Constructor
  const PrimaryKey(DataType dataType, String dataClass)
      : super(dataType, dataClass);

  /// Construct from JSON data.
  factory PrimaryKey.fromPds(Map<String, dynamic> data) {
    return new PrimaryKey(
        DataType.values[data['type'] as int], data['class'] as String);
  }

  /// Convert to JSON compatible data.
  Map<String, dynamic> toPds() {
    return {'type': dataType.index, 'class': dataClass};
  }
}
