// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of molview_dataserver.msglib;

/// Which snippets to retrieve?
enum WhichSnippets {
  /// - One to one relation: return one snippet via the most efficient retrieval
  ///   path.
  /// - Many to * relation: return all available snippets via the most efficient
  ///   retrieval path.
  auto,

  /// Retrieve all available snippets from all providing tables. This option is
  /// generally only interesting when you are qualitatively comparing databases.
  all
}

/// Snippet selector
///
/// A snippet is a piece of data with a specific data class. This data snippet
/// can be retrieved from any table that contains a field that matches this
/// snippet. You can also force an origin database and table.
class SnippetSelector {
  /// Origin database
  final String database;

  /// Origin table
  final String table;

  /// Retrieve all matching fields
  ///
  /// There are different interpretations for this: you can retrieve this
  /// snippet from all tables that provide it, or you can
  final WhichSnippets which;

  /// Desired field data class
  final String dataClass;

  /// Desired field attributes
  final Map<String, String> attributes;

  /// Input parameters
  final Map<String, String> parameters;

  /// Constructor
  SnippetSelector(this.dataClass,
      {this.database: '',
      this.table: '',
      this.which: WhichSnippets.auto,
      this.attributes: const {},
      this.parameters: const {}});

  /// Construct from PDS.
  factory SnippetSelector.fromPds(Map<String, dynamic> data) {
    return new SnippetSelector(data['class'] as String,
        database: data['database'] as String,
        table: data['table'] as String,
        which: WhichSnippets.values[data['which'] as int],
        attributes: data['attributes'] as Map<String, String>,
        parameters: data['parameters'] as Map<String, String>);
  }

  /// Convert to PDS.
  Map<String, dynamic> toPds() {
    return {
      'database': database,
      'table': table,
      'class': dataClass,
      'which': which.index,
      'attributes': pdsFromMap(attributes),
      'parameters': pdsFromMap(parameters)
    };
  }
}

/// Snippet data result from a specific table
class SnippetResult {
  /// Origin database
  final String database;

  /// Origin table
  final String table;

  /// All values that are in this table
  final List values;

  /// Constructor
  SnippetResult(this.database, this.table, this.values);

  /// Construct from PDS.
  factory SnippetResult.fromPds(Map<String, dynamic> data) {
    return new SnippetResult(
        data['database'] as String, data['table'] as String, data['values']);
  }

  /// Convert to PDS.
  Map<String, dynamic> toPds() {
    return {'database': database, 'table': table, 'values': values};
  }
}

/// Snippet data
class SnippetData {
  /// Data class of this snippet
  final String dataClass;

  /// All retrieved results
  final List<SnippetResult> results;

  /// Constructor
  SnippetData(this.dataClass, this.results);

  /// Construct from PDS.
  factory SnippetData.fromPds(Map<String, dynamic> data) {
    var r = data['results'] as List;
    return new SnippetData(
        data['class'] as String,
        new List<SnippetResult>.generate(
            r.length,
            (int i) =>
                new SnippetResult.fromPds(r[i] as Map<String, dynamic>)));
  }

  /// Convert to PDS.
  Map<String, dynamic> toPds() {
    return {'class': dataClass, 'results': pdsFromList(results)};
  }
}
